﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Wpf_CopyScreen
{
    /// <summary>
    /// WindowCopy.xaml 的交互逻辑
    /// </summary>
    public partial class WindowCopy : Window
    {
        private ImageSource ISource;
        private int width;
        private int height;
        private Color mainColor;
        //private Point mousePoint;
        private Point firstPoint = new Point(0,0);
        private Point lastPoint = new Point(0,0);
        private bool IsMouseDown_T = false;
        private bool IsMouseDown_R = false;
        private bool IsMouseDown_B = false;
        private bool IsMouseDown_L = false;
        //private bool IsMouseDown_Copy = false;
        private bool IsShouldChange = true;

        public WindowMain winMain;

        public WindowCopy(ImageSource ISource)
        {
            InitializeComponent();
            this.ISource = ISource;
            this.mainColor = Color.FromArgb((byte)128, (byte)0, (byte)0, (byte)0);
            this.width = (int)SystemParameters.PrimaryScreenWidth;
            this.height = (int)SystemParameters.PrimaryScreenHeight;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            this.Background = new ImageBrush(ISource);
            polygonCopy.Points = InitPoints(firstPoint,lastPoint);
            polygonCopy.Fill = new SolidColorBrush(mainColor);
        }

        /// <summary>
        /// 鼠标按下时
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if (e.LeftButton == MouseButtonState.Pressed)
            {
                if (rectangleCopy.Width == 0 || rectangleCopy.Height == 0)
                {
                    firstPoint = e.GetPosition(this);
                }
                else
                {
                    IsShouldChange = false;

                    if (rectLT.IsMouseOver)
                    {
                        IsShouldChange = true;
                        firstPoint = new Point(rectangleCopy.Margin.Left + rectangleCopy.Width, rectangleCopy.Margin.Top + rectangleCopy.Height);
                    }
                    else if (rectT.IsMouseOver)
                    {
                        IsShouldChange = true;
                        IsMouseDown_T = true;
                        firstPoint = new Point(rectangleCopy.Margin.Left, rectangleCopy.Margin.Top + rectangleCopy.Height);
                    }
                    else if (rectTR.IsMouseOver)
                    {
                        IsShouldChange = true;
                        firstPoint = new Point(rectangleCopy.Margin.Left, rectangleCopy.Margin.Top + rectangleCopy.Height);
                    }
                    else if (rectR.IsMouseOver)
                    {
                        IsShouldChange = true;
                        IsMouseDown_R = true;
                        firstPoint = new Point(rectangleCopy.Margin.Left, rectangleCopy.Margin.Top);
                    }
                    else if (rectRB.IsMouseOver)
                    {
                        IsShouldChange = true;
                        firstPoint = new Point(rectangleCopy.Margin.Left, rectangleCopy.Margin.Top);
                    }
                    else if (rectB.IsMouseOver)
                    {
                        IsShouldChange = true;
                        IsMouseDown_B = true;
                        firstPoint = new Point(rectangleCopy.Margin.Left + rectangleCopy.Width, rectangleCopy.Margin.Top);
                    }
                    else if (rectBL.IsMouseOver)
                    {
                        IsShouldChange = true;
                        firstPoint = new Point(rectangleCopy.Margin.Left + rectangleCopy.Width, rectangleCopy.Margin.Top);
                    }
                    else if (rectL.IsMouseOver)
                    {
                        IsShouldChange = true;
                        IsMouseDown_L = true;
                        firstPoint = new Point(rectangleCopy.Margin.Left + rectangleCopy.Width, rectangleCopy.Margin.Top + rectangleCopy.Height);
                    }
                    //else if (rectangleCopy.IsMouseOver)
                    //{
                    //    IsShouldChange = true;
                    //    IsMouseDown_Copy = true;
                    //    mousePoint = e.GetPosition(this);
                    //}
                }
            }
            if (e.RightButton == MouseButtonState.Pressed)
            {
                if (rectangleCopy.Width == 0 || rectangleCopy.Height == 0)
                {
                    this.Close();
                    winMain.Show();
                }
                else
                {
                    rectangleCopy.Width = 0;
                    rectangleCopy.Height = 0;
                    firstPoint = lastPoint = new Point(0,0);

                    SetAll();
                }
                IsShouldChange = true;
            }
        }

        /// <summary>
        /// 鼠标在窗体上移动时
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.LeftButton == MouseButtonState.Pressed)
            {
                lastPoint = e.GetPosition(this);

                if (rectT.IsMouseOver || IsMouseDown_T)
                {
                    lastPoint = new Point(rectangleCopy.Margin.Left + rectangleCopy.Width, lastPoint.Y);
                }
                if (rectR.IsMouseOver || IsMouseDown_R)
                {
                    lastPoint = new Point(lastPoint.X, rectangleCopy.Margin.Top + rectangleCopy.Height);
                }
                if (rectB.IsMouseOver || IsMouseDown_B)
                {
                    lastPoint = new Point(rectangleCopy.Margin.Left, lastPoint.Y);
                }
                if (rectL.IsMouseOver || IsMouseDown_L)
                {
                    lastPoint = new Point(lastPoint.X, rectangleCopy.Margin.Top);
                }
                //if (rectangleCopy.IsMouseOver || IsMouseDown_Copy)
                //{
                //    double moveX = e.GetPosition(this).X - mousePoint.X;
                //    double moveY = e.GetPosition(this).Y - mousePoint.Y;
                //    firstPoint = new Point(rectangleCopy.Margin.Left + moveX,rectangleCopy.Margin.Top + moveY);
                //    lastPoint = new Point(rectangleCopy.Margin.Left + rectangleCopy.Width + moveX, rectangleCopy.Margin.Top + rectangleCopy.Height + moveY);

                //    mousePoint = e.GetPosition(this);
                //}

                if (IsShouldChange)
                {
                    SetAll();
                }
            }
        }

        /// <summary>
        /// 鼠标释放时
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_MouseUp(object sender, MouseButtonEventArgs e)
        {
            if (e.LeftButton == MouseButtonState.Released)
            {
                IsMouseDown_T = false;
                IsMouseDown_R = false;
                IsMouseDown_B = false;
                IsMouseDown_L = false;
            }
        }

        private void Window_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            Rect rect = new Rect(rectangleCopy.Margin.Left, rectangleCopy.Margin.Top, rectangleCopy.Width, rectangleCopy.Height);
            ImageSource ISource = CopyHelper.BitMapToImageSource(CopyHelper.CutPicture(rect));
            winMain.rectImage.Fill = new ImageBrush(ISource);
            winMain.rectImage.Width = rectangleCopy.Width;
            winMain.rectImage.Height = rectangleCopy.Height;
            
            this.Close();
            winMain.Show();
        }

        /// <summary>
        /// 移动矩形选框MouseDown
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        //private void rectangleCopy_MouseDown(object sender, MouseButtonEventArgs e)
        //{
        //    if (e.LeftButton == MouseButtonState.Pressed)
        //    {
        //        mousePoint = e.GetPosition(null);
        //    }
        //}

        /// <summary>
        /// 移动矩形选框MouseMove
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        //private void rectangleCopy_MouseMove(object sender, MouseEventArgs e)
        //{
        //    if (e.LeftButton == MouseButtonState.Pressed)
        //    {
        //        Point tempPoint = e.GetPosition(this);
        //        double deltaX = tempPoint.X - mousePoint.X;
        //        double deltaY = tempPoint.Y - mousePoint.Y;

        //        rectangleCopy.Margin = new Thickness(rectangleCopy.Margin.Left + deltaX, rectangleCopy.Margin.Top + deltaY, 0, 0);
        //        mousePoint = e.GetPosition(null);

        //        firstPoint = new Point(rectangleCopy.Margin.Left, rectangleCopy.Margin.Top);
        //        lastPoint = new Point(rectangleCopy.Margin.Left + rectangleCopy.Width, rectangleCopy.Margin.Top + rectangleCopy.Height);

        //        SetAll();
        //    }
        //}

        /// <summary>
        /// 
        /// </summary>
        private void SetAll()
        {
            SetSPMsg(firstPoint, lastPoint);
            SetBorderAndRect(firstPoint, lastPoint);
            polygonCopy.Points = InitPoints(firstPoint, lastPoint);
        }

        /// <summary>
        /// 根据矩形选择框的两个相对的端点来初始化Polygon的Points
        /// </summary>
        /// <param name="firstPoint"></param>
        /// <param name="lastPoint"></param>
        /// <returns></returns>
        private PointCollection InitPoints(Point firstPoint, Point lastPoint)
        {
            double minX = Math.Min(firstPoint.X, lastPoint.X);
            double minY = Math.Min(firstPoint.Y, lastPoint.Y);
            double maxX = Math.Max(firstPoint.X, lastPoint.X);
            double maxY = Math.Max(firstPoint.Y, lastPoint.Y);

            List<Point> listPoints = new List<Point>();
            listPoints.Add(new Point(0, 0));
            listPoints.Add(new Point(width, 0));
            listPoints.Add(new Point(width, height));
            listPoints.Add(new Point(0, height));
            listPoints.Add(new Point(0, 0));
            listPoints.Add(new Point(minX, minY));
            listPoints.Add(new Point(maxX, minY));
            listPoints.Add(new Point(maxX, maxY));
            listPoints.Add(new Point(minX, maxY));
            listPoints.Add(new Point(minX, minY));

            return new PointCollection(listPoints);
        }

        /// <summary>
        /// 显示截图时的信息
        /// </summary>
        /// <param name="firstPoint"></param>
        /// <param name="lastPoint"></param>
        private void SetSPMsg(Point firstPoint, Point lastPoint)
        {
            if (rectangleCopy.Width == 0 || rectangleCopy.Height == 0)
            {
                SPMsg.Visibility = Visibility.Hidden;
            }
            else
            {
                SPMsg.Visibility = Visibility.Visible;
                Point tempPoint = Mouse.GetPosition(this);
                Color tempColor = CopyHelper.GetColor(tempPoint);
                txtSize.Text = "区域大小：" + rectangleCopy.Width.ToString() + " * " + rectangleCopy.Height.ToString();
                txtPos.Text = "鼠标位置：(" + tempPoint.X.ToString() + "," + tempPoint.Y.ToString() + ")";
                txtRGB.Text = "当前RGB：(" + tempColor.R.ToString() + "," + tempColor.G.ToString() + "," + tempColor.B + ")"; ;

                double minX = Math.Min(firstPoint.X, lastPoint.X);
                double minY = Math.Min(firstPoint.Y, lastPoint.Y);

                if (minY < SPMsg.Height + 8)
                {
                    SPMsg.Margin = new Thickness(minX, minY + 8, 0, 0);
                }
                else
                {
                    SPMsg.Margin = new Thickness(minX, minY - SPMsg.Height - 8, 0, 0);
                }
            }
        }

        /// <summary>
        /// 根据矩形选择框的两个相对的端点来设置矩形选框的颜色及矩形选框顶点及边上小矩形的位置
        /// </summary>
        /// <param name="firstPoint"></param>
        /// <param name="lastPoint"></param>
        private void SetBorderAndRect(Point firstPoint, Point lastPoint)
        {
            double minX = Math.Min(firstPoint.X, lastPoint.X);
            double minY = Math.Min(firstPoint.Y, lastPoint.Y);
            double maxX = Math.Max(firstPoint.X, lastPoint.X);
            double maxY = Math.Max(firstPoint.Y, lastPoint.Y);

            rectangleCopy.Margin = new Thickness(minX, minY, 0,0);
            rectangleCopy.Width = maxX - minX;
            rectangleCopy.Height = maxY - minY;

            if (rectangleCopy.Width == 0 || rectangleCopy.Height == 0)
            {
                rectLT.Visibility = Visibility.Hidden;
                rectT.Visibility = Visibility.Hidden;
                rectTR.Visibility = Visibility.Hidden;
                rectR.Visibility = Visibility.Hidden;
                rectRB.Visibility = Visibility.Hidden;
                rectB.Visibility = Visibility.Hidden;
                rectBL.Visibility = Visibility.Hidden;
                rectL.Visibility = Visibility.Hidden;
            }
            else
            {
                rectLT.Visibility = Visibility.Visible;
                rectT.Visibility = Visibility.Visible;
                rectTR.Visibility = Visibility.Visible;
                rectR.Visibility = Visibility.Visible;
                rectRB.Visibility = Visibility.Visible;
                rectB.Visibility = Visibility.Visible;
                rectBL.Visibility = Visibility.Visible;
                rectL.Visibility = Visibility.Visible;
            }

            rectLT.Margin = new Thickness(minX - 2, minY - 2,0,0);
            rectT.Margin = new Thickness(minX - 2 + rectangleCopy.Width / 2, minY - 2, 0, 0);
            rectTR.Margin = new Thickness(maxX - 3, minY - 2, width - maxX - 2, height - maxY - 2);
            rectR.Margin = new Thickness(maxX - 3, minY - 2 + rectangleCopy.Height / 2, 0, 0);
            rectRB.Margin = new Thickness(maxX - 3, maxY - 3, 0, 0);
            rectB.Margin = new Thickness(minX - 2 + rectangleCopy.Width / 2, maxY - 3, 0, 0);
            rectBL.Margin = new Thickness(minX - 2, maxY - 3, 0, 0);
            rectL.Margin = new Thickness(minX - 2, minY - 2 + rectangleCopy.Height / 2, 0, 0);
        }
    }
}